@@ -25,8 +25,10 @@ An emoji picker for ibus typing booster.
 from typing import Any
 from typing import List
 from typing import Tuple
+from typing import Iterable
 from typing import Dict
 from typing import Optional
+from typing import Callable
 import sys
 import os
 import re
@@ -67,8 +69,8 @@ GTK_VERSION = (Gtk.get_major_version(),
                Gtk.get_micro_version())
 
 DOMAINNAME = 'ibus-typing-booster'
-_ = lambda a: gettext.dgettext(DOMAINNAME, a)
-N_ = lambda a: a
+_: Callable[[str], str] = lambda a: gettext.dgettext(DOMAINNAME, a)
+N_: Callable[[str], str] = lambda a: a
 
 def parse_args() -> Any:
     '''
@@ -185,19 +187,19 @@ def parse_args() -> Any:
 
 _ARGS = parse_args()
 
-class EmojiPickerUI(Gtk.Window):
+class EmojiPickerUI(Gtk.Window): # type: ignore
     '''
     User interface of the emoji picker
     '''
     def __init__(self,
-                 languages=('en_US',),
-                 modal=False,
-                 unicode_data_all=False,
-                 emoji_unicode_min=1.0,
-                 emoji_unicode_max=100.0,
-                 font=None,
-                 fontsize=None,
-                 fallback=None) -> None:
+                 languages: Iterable[str] = ('en_US',),
+                 modal: bool = False,
+                 unicode_data_all: bool = False,
+                 emoji_unicode_min: str = '1.0',
+                 emoji_unicode_max: str = '100.0',
+                 font: Optional[str] = None,
+                 fontsize: Optional[float] = None,
+                 fallback: Optional[bool] = None) -> None:
         Gtk.Window.__init__(self, title='🚀 ' + _('Emoji Picker'))
 
         self.set_name('EmojiPicker')
@@ -536,9 +538,9 @@ class EmojiPickerUI(Gtk.Window):
         if _ARGS.debug:
             LOGGER.debug(
                 'natural_width_search_entry = %s '
-                %natural_width_search_entry
-                + 'natural_width_search_bar = %s\n'
-                %natural_width_search_bar)
+                'natural_width_search_bar = %s\n',
+                natural_width_search_entry,
+                natural_width_search_bar)
         self._browse_paned.set_position(natural_width_search_bar)
 
         self._selection_clipboard = Gtk.Clipboard.get(
@@ -634,7 +636,7 @@ class EmojiPickerUI(Gtk.Window):
             N_('Keywords'),
         ]
         if self._gettext_translations[language]:
-            return self._gettext_translations[language].gettext(key)
+            return str(self._gettext_translations[language].gettext(key))
         return key
 
     def _emoji_descriptions(self, emoji: str) -> List[str]:
@@ -648,10 +650,10 @@ class EmojiPickerUI(Gtk.Window):
         '''
         descriptions = []
         descriptions.append(
-            ' '.join(['U+%X' %ord(character) for character in emoji]))
+            ' '.join([f'U+{ord(character):04X}' for character in emoji]))
         for language in itb_util.expand_languages(self._languages):
             names = self._emoji_matcher.names(emoji, language=language)
-            description = '<b>%s</b>' %language
+            description = f'<b>{language}</b>'
             description_empty = True
             if names:
                 description += '\n' + html.escape(', '.join(names))
@@ -678,12 +680,12 @@ class EmojiPickerUI(Gtk.Window):
             fonts_description += '\n'
             code_points = ''
             for char in text:
-                code_points += ' U+%X' %ord(char)
+                code_points += f' U+{ord(char):04X}'
             fonts_description += (
-                '<span font="%s" fallback="%s" >'
-                %(self._font, str(self._fallback).lower())
+                f'<span font="{self._font}" '
+                f'fallback="{str(self._fallback).lower()}" >'
                 + text + '</span>'
-                + '<span fallback="true">%s</span>' % code_points)
+                + f'<span fallback="true">{code_points}</span>')
             fonts_description += ': ' + font_family
         descriptions.append(fonts_description)
         if self._emoji_matcher.unicode_version(emoji):
@@ -692,9 +694,9 @@ class EmojiPickerUI(Gtk.Window):
                 + self._emoji_matcher.unicode_version(emoji))
         if _ARGS.debug:
             descriptions.append(
-                'emoji_order = %s' %self._emoji_matcher.emoji_order(emoji))
+                f'emoji_order = {self._emoji_matcher.emoji_order(emoji)}')
             descriptions.append(
-                'cldr_order = %s' %self._emoji_matcher.cldr_order(emoji))
+                f'cldr_order = {self._emoji_matcher.cldr_order(emoji)}')
             descriptions.append(
                 'Emoji properties from unicode.org:' + '\n'
                 + ', '.join(self._emoji_matcher.properties(emoji)))
@@ -801,15 +803,14 @@ class EmojiPickerUI(Gtk.Window):
             gtk_label = Gtk.Label()
             # Make font for emoji large using pango markup
             text = (
-                '<span font="%s %s" fallback="%s">'
-                %(self._font, self._fontsize, str(self._fallback).lower())
+                f'<span font="{self._font} {self._fontsize}" '
+                f'fallback="{str(self._fallback).lower()}">'
                 + html.escape(emoji)
                 + '</span>')
             if itb_emoji.is_invisible(emoji):
                 text += (
-                    '<span fallback="false" font="%s">'
-                    %(self._fontsize / 2)
-                    + ' U+%X ' %ord(emoji)
+                    f'<span fallback="false" font="{self._fontsize / 2}">'
+                    f' U+{ord(emoji):04X} '
                     + self._emoji_matcher.name(emoji)
                     + '</span>')
             gtk_label.set_text(text)
@@ -883,10 +884,13 @@ class EmojiPickerUI(Gtk.Window):
                           mode='r',
                           encoding='UTF-8') as options_file:
                     options_dict = eval(options_file.read())
-            except (PermissionError, SyntaxError, IndentationError):
-                LOGGER.exception('Error when reading options')
-            except Exception as dummy_exception:
-                LOGGER.exception('Unknown error when reading options')
+            except (PermissionError, SyntaxError, IndentationError) as error:
+                LOGGER.exception('Error when reading options: %s: %s',
+                                 error.__class__.__name__, error)
+            except Exception as error:
+                LOGGER.exception(
+                    'Unexpected error when reading options: %s: %s',
+                    error.__class__.__name__, error)
             else: # no exception occured
                 if _ARGS.debug:
                     LOGGER.debug(
@@ -946,12 +950,13 @@ class EmojiPickerUI(Gtk.Window):
                           mode='r',
                           encoding='UTF-8') as recently_used_file:
                     recently_used_emoji = eval(recently_used_file.read())
-            except (PermissionError, SyntaxError, IndentationError):
-                import traceback
-                traceback.print_exc()
-            except Exception as dummy_exception:
-                import traceback
-                traceback.print_exc()
+            except (PermissionError, SyntaxError, IndentationError) as error:
+                LOGGER.exception('Error reading recently used emoji: %s: %s',
+                                 error.__class__.__name__, error)
+            except Exception as error:
+                LOGGER.exception(
+                    'Unexpected error reading recently used emoji: %s: %s',
+                    error.__class__.__name__, error)
             else: # no exception occured
                 if _ARGS.debug:
                     LOGGER.debug(
@@ -974,7 +979,7 @@ class EmojiPickerUI(Gtk.Window):
             self._init_recently_used()
         self._cleanup_recently_used()
 
-    def _cleanup_recently_used(self):
+    def _cleanup_recently_used(self) -> None:
         '''
         Reduces the size of the recently used dictionary
         if it has become too big.
@@ -1082,7 +1087,8 @@ class EmojiPickerUI(Gtk.Window):
                     + ', '.join(keywords))
                 break
 
-    def _print_profiling_information(self) -> None:
+    @staticmethod
+    def _print_profiling_information() -> None:
         '''
         Print some profiling information to the log.
         '''
@@ -1095,7 +1101,7 @@ class EmojiPickerUI(Gtk.Window):
         stats.print_stats('itb_emoji', 25)
         LOGGER.info('Profiling info:\n%s', stats_stream.getvalue())
 
-    def on_delete_event(self, *_args) -> None:
+    def on_delete_event(self, *_args: Any) -> None:
         '''
         The window has been deleted, probably by the window manager.
         '''
@@ -1104,7 +1110,7 @@ class EmojiPickerUI(Gtk.Window):
             self._print_profiling_information()
         Gtk.main_quit()
 
-    def on_destroy_event(self, *_args) -> None:
+    def on_destroy_event(self, *_args: Any) -> None:
         '''
         The window has been destroyed.
         '''
@@ -1225,12 +1231,11 @@ class EmojiPickerUI(Gtk.Window):
             label = Gtk.Label()
             # Make font for emoji large using pango markup
             label.set_text(
-                '<span font="%s %s" fallback="%s">'
-                %(self._font, self._fontsize, str(self._fallback).lower())
+                f'<span font="{self._font} {self._fontsize}" '
+                f'fallback="{str(self._fallback).lower()}">'
                 + html.escape(emoji)
                 + '</span>'
-                + '<span font="%s">'
-                %(self._fontsize / 2)
+                + f'<span font="{self._fontsize / 2}">'
                 + ' ' + html.escape(name)
                 + '</span>')
             label.set_use_markup(True)
@@ -1371,10 +1376,10 @@ class EmojiPickerUI(Gtk.Window):
             return
         if _ARGS.debug:
             LOGGER.debug(
-                'on_label_selected(): model[iterator] = %s\n'
-                %model[iterator][:]
-                + 'self._currently_selected_label = %s\n'
-                %repr(self._currently_selected_label))
+                'model[iterator] = %s\n'
+                'self._currently_selected_label = %s\n',
+                model[iterator][:],
+                repr(self._currently_selected_label))
         self._search_bar.set_search_mode(False)
         self._browse_treeview.grab_focus()
         language = model[iterator][1]
@@ -1439,7 +1444,7 @@ class EmojiPickerUI(Gtk.Window):
             self._emoji_selected_popover = None
         return False
 
-    def _emoji_event_box_selected(self, event_box) -> bool:
+    def _emoji_event_box_selected(self, event_box: Gtk.EventBox) -> bool:
         '''
         Called when an event box containing an emoji
         was selected in the flowbox.
@@ -1449,7 +1454,6 @@ class EmojiPickerUI(Gtk.Window):
         has been copied to the clipboard.
 
         :param event_box: The event box which contains the emoji
-        :type event_box: Gtk.EventBox object
         '''
         # Use .get_label() instead of .get_text() to fetch the text
         # from the label widget including any embedded underlines
@@ -1461,7 +1465,8 @@ class EmojiPickerUI(Gtk.Window):
             LOGGER.debug('text = %s\n', text)
         (emoji, name) = self._parse_emoji_and_name_from_text(text)
         if not emoji:
-            return Gdk.EVENT_PROPAGATE
+            # Gdk.EVENT_PROPAGATE is defined as False
+            return bool(Gdk.EVENT_PROPAGATE)
         self._show_concise_emoji_description_in_header_bar(emoji)
         self._set_clipboards(emoji)
         self._add_to_recently_used(emoji)
@@ -1540,14 +1545,13 @@ class EmojiPickerUI(Gtk.Window):
             if emoji:
                 emoji = self._variation_selector_normalize_for_font(emoji)
                 new_text = (
-                    '<span font="%s %s" fallback="%s">'
-                    %(self._font, self._fontsize, str(self._fallback).lower())
+                    f'<span font="{self._font} {self._fontsize}" '
+                    f'fallback="{str(self._fallback).lower()}">'
                     + html.escape(emoji)
                     + '</span>')
                 if name:
                     new_text += (
-                        '<span fallback="true" font="%s">'
-                        %(self._fontsize / 2)
+                        f'<span fallback="true" font="{self._fontsize / 2}">'
                         + html.escape(name)
                         + '</span>')
                 label.set_text(new_text)
@@ -1624,16 +1628,16 @@ class EmojiPickerUI(Gtk.Window):
             # in a search results flowbox and we do *not* want
             # to replace the emoji.
             new_text = (
-                '<span font="%s %s" fallback="%s">'
-                %(self._font, self._fontsize, str(self._fallback).lower())
+                f'<span font="{self._font} {self._fontsize}" '
+                f'fallback="{str(self._fallback).lower()}">'
                 + html.escape(emoji)
                 + '</span>')
             label.set_text(new_text)
             label.set_use_markup(True)
         GLib.timeout_add(500, self._skin_tone_selected_popover_popdown)
 
+    @staticmethod
     def on_flowbox_event_box_long_press_begin(
-            self,
             gesture: Gtk.GestureLongPress,
             event_sequence: Gdk.EventSequence) -> None:
         '''
@@ -1643,12 +1647,10 @@ class EmojiPickerUI(Gtk.Window):
         :param event_sequence: the event sequence that the event belongs to
         '''
         if _ARGS.debug:
-            LOGGER.debug(
-                'on_flowbox_event_box_long_press_begin() %s %s\n',
-                repr(gesture), repr(event_sequence))
+            LOGGER.debug('%s %s\n', repr(gesture), repr(event_sequence))
 
+    @staticmethod
     def on_flowbox_event_box_long_press_cancel(
-            self,
             gesture: Gtk.GestureLongPress,
             event_sequence: Gdk.EventSequence) -> None:
         '''
@@ -1659,11 +1661,11 @@ class EmojiPickerUI(Gtk.Window):
         '''
         if _ARGS.debug:
             LOGGER.debug(
-                'on_flowbox_event_box_long_press_cancel() %s %s\n',
-                repr(gesture), repr(event_sequence))
+                '%s %s\n', repr(gesture), repr(event_sequence))
 
+    @staticmethod
     def on_flowbox_event_box_long_press_cancelled(
-            self, gesture: Gtk.GestureLongPress) -> None:
+            gesture: Gtk.GestureLongPress) -> None:
         '''
         Signal handler called whenever a press moved too far, or was
         released before “pressed” happened.
@@ -1671,9 +1673,7 @@ class EmojiPickerUI(Gtk.Window):
         :param gesture: The object which received the signal
         '''
         if _ARGS.debug:
-            LOGGER.debug(
-                'on_flowbox_event_box_long_press_cancelled() %s\n',
-                repr(gesture))
+            LOGGER.debug('%s\n', repr(gesture))
 
     def on_flowbox_event_box_long_press_pressed(
             self,
@@ -1707,10 +1707,10 @@ class EmojiPickerUI(Gtk.Window):
         '''
         if _ARGS.debug:
             LOGGER.debug(
-                'on_flowbox_event_box_button_release() '
-                + 'event_button.type=%s ' %event_button.type
-                + 'event_button.window=%s ' %event_button.window
-                + 'event_button.button=%s\n' %event_button.button)
+                'event_button.type=%s '
+                'event_button.window=%s '
+                'event_button.button=%s\n',
+                event_button.type, event_button.window, event_button.button)
         if event_button.button == 1:
             # Call self._emoji_event_box_selected() in the release
             # event because here we know that it has not been a long
@@ -1800,8 +1800,8 @@ class EmojiPickerUI(Gtk.Window):
         for skin_tone_variant in skin_tone_variants:
             label = Gtk.Label()
             label.set_text(
-                '<span font="%s %s" fallback="%s">'
-                %(self._font, self._fontsize, str(self._fallback).lower())
+                f'<span font="{self._font} {self._fontsize}" '
+                f'fallback="{str(self._fallback).lower()}">'
                 + html.escape(skin_tone_variant)
                 + '</span>')
             label.set_use_markup(True)
@@ -1908,8 +1908,8 @@ class EmojiPickerUI(Gtk.Window):
         label.set_vexpand(False)
         label.set_halign(Gtk.Align.START)
         label.set_markup(
-            '<span font="%s %s" fallback="%s">'
-            %(self._font, self._fontsize * 3, str(self._fallback).lower())
+            f'<span font="{self._font} {self._fontsize * 3}" '
+            f'fallback="{str(self._fallback).lower()}">'
             + html.escape(emoji)
             + '</span>')
         emoji_info_popover_listbox.insert(label, -1)
@@ -1978,10 +1978,10 @@ class EmojiPickerUI(Gtk.Window):
         '''
         if _ARGS.debug:
             LOGGER.debug(
-                'on_flowbox_event_box_button_press() '
-                + 'event_button.type=%s ' %event_button.type
-                + 'event_button.window=%s ' %event_button.window
-                + 'event_button.button=%s\n' %event_button.button)
+                'event_button.type=%s '
+                'event_button.window=%s '
+                'event_button.button=%s\n',
+                event_button.type, event_button.window, event_button.button)
         if event_button.type != Gdk.EventType.BUTTON_PRESS:
             # ignore double and triple clicks i.e. ignore
             # Gdk.EventType.2BUTTON_PRESS and
@@ -2220,7 +2220,7 @@ def get_languages() -> List[str]:
     If not, get it from the environment variables.
     '''
     if _ARGS.languages:
-        return _ARGS.languages.split(':')
+        return list(_ARGS.languages.split(':'))
     environment_variable = os.getenv('LANGUAGE')
     if not environment_variable:
         environment_variable = os.getenv('LC_ALL')
