@@ -19,71 +19,79 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>
 
-
+from typing import Dict
+from typing import Optional
+from typing import Callable
 import re
 import os
-import sys
+import logging
 from gettext import dgettext
-_ = lambda a: dgettext("ibus-typing-booster", a)
-N_ = lambda a: a
-from gi import require_version
+_: Callable[[str], str] = lambda a: dgettext("ibus-typing-booster", a)
+N_: Callable[[str], str] = lambda a: a
+from gi import require_version # type: ignore
 require_version('IBus', '1.0')
-from gi.repository import IBus
+from gi.repository import IBus # type: ignore
 import hunspell_table
 import tabsqlitedb
 
+LOGGER = logging.getLogger('ibus-typing-booster')
+
 DEBUG_LEVEL = int(0)
 
-class EngineFactory(IBus.Factory):
+class EngineFactory(IBus.Factory): # type: ignore
     """Table IM Engine Factory"""
-    def __init__(self, bus):
+    def __init__(self, bus: IBus.Bus) -> None:
         global DEBUG_LEVEL
         try:
-            DEBUG_LEVEL = int(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL'))
+            DEBUG_LEVEL = int(
+                str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
         except (TypeError, ValueError):
             DEBUG_LEVEL = int(0)
         if DEBUG_LEVEL > 1:
-            sys.stderr.write("EngineFactory.__init__(bus=%s)\n" %bus)
-        self.db = None
-        self.dbdict = {}
-        self.enginedict = {}
+            LOGGER.debug('EngineFactory.__init__(bus=%s)\n', bus)
+        self.database: Optional[tabsqlitedb.TabSqliteDb] = None
+        self.database_dict: Dict[str, tabsqlitedb.TabSqliteDb] = {}
+        self.enginedict: Dict[str, hunspell_table.TypingBoosterEngine] = {}
         self.bus = bus
         #engine.Engine.CONFIG_RELOADED(bus)
-        super(EngineFactory, self).__init__(
+        super().__init__(
             connection=bus.get_connection(), object_path=IBus.PATH_FACTORY)
         self.engine_id = 0
 
-    def do_create_engine(self, engine_name):
+    def do_create_engine(
+            self, engine_name: str) -> hunspell_table.TypingBoosterEngine:
         if DEBUG_LEVEL > 1:
-            sys.stderr.write("EngineFactory.do_create_engine(engine_name=%s)\n"
-                             % engine_name)
+            LOGGER.debug(
+                'EngineFactory.do_create_engine(engine_name=%s)\n',
+                engine_name)
         engine_base_path = "/com/redhat/IBus/engines/table/%s/engine/"
         engine_path = engine_base_path % re.sub(
             r'[^a-zA-Z0-9_/]', '_', engine_name)
         try:
-            if engine_name in self.dbdict:
-                self.db = self.dbdict[engine_name]
+            if engine_name in self.database_dict:
+                self.database = self.database_dict[engine_name]
             else:
-                self.db = tabsqlitedb.TabSqliteDb()
-                self.dbdict[engine_name] = self.db
+                self.database = tabsqlitedb.TabSqliteDb()
+                self.database_dict[engine_name] = self.database
             if engine_name in self.enginedict:
                 engine = self.enginedict[engine_name]
             else:
                 engine = hunspell_table.TypingBoosterEngine(
-                    self.bus, engine_path + str(self.engine_id), self.db)
+                    self.bus, engine_path + str(self.engine_id), self.database)
                 self.enginedict[engine_name] = engine
                 self.engine_id += 1
             return engine
-        except:
-            print("failed to create engine %s" %engine_name)
+        except Exception as error:
+            print(f'failed to create engine {engine_name}: '
+                  f'{error.__class__.__name__}: {error}')
             import traceback
             traceback.print_exc()
-            raise Exception("Cannot create engine %s" % engine_name)
+            raise Exception from error
 
-    def do_destroy(self):
+    def do_destroy(self) -> None:
         '''Destructor, which finish some task for IME'''
-        if DEBUG_LEVEL > 1:
-            sys.stderr.write("EngineFactory.do_destroy()\n")
-        for _db in self.dbdict:
-            self.dbdict[_db].sync_usrdb()
-        super(EngineFactory, self).destroy()
+        LOGGER.info('Syncing user database(s)')
+        for key, database in self.database_dict.items():
+            LOGGER.info('Syncing %s %s', key, database)
+            database.sync_usrdb()
+        super().destroy()
