@@ -27,6 +27,7 @@ hunspell dictonaries.
 from typing import Dict
 from typing import Tuple
 from typing import List
+from typing import Iterable
 import os
 import sys
 import unicodedata
@@ -79,7 +80,7 @@ MAX_WORDS = 100
 class Dictionary:
     '''A class to hold a hunspell dictionary
     '''
-    def __init__(self, name='en_US') -> None:
+    def __init__(self, name: str = 'en_US') -> None:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('Dictionary.__init__(name=%s)\n', name)
         self.name = name
@@ -126,33 +127,41 @@ class Dictionary:
                     LOGGER.warning(
                         'Language is “fi” but “import libvoikko” failed.')
                     return
-                self.voikko = libvoikko.Voikko('fi')
+                try:
+                    self.voikko = libvoikko.Voikko('fi')
+                except (libvoikko.VoikkoException,) as error:
+                    LOGGER.warning('Init of voikko failed: %s: %s',
+                                   error.__class__.__name__, error)
+                    self.voikko = None
                 return
             if IMPORT_ENCHANT_SUCCESSFUL:
                 try:
                     self.enchant_dict = enchant.Dict(self.name)
-                except enchant.errors.DictNotFoundError:
+                except enchant.errors.DictNotFoundError as error:
                     LOGGER.exception(
-                        'Error initializing enchant for %s', self.name)
+                        'Error initializing enchant for %s: %s: %s',
+                        self.name, error.__class__.__name__, error)
                     self.enchant_dict = None
-                except Exception:
+                except Exception as error:
                     LOGGER.exception(
-                        'Unknown error initializing enchant for %s',
-                        self.name)
+                        'Unexpected error initializing enchant for %s: %s: %s',
+                        self.name, error.__class__.__name__, error)
                     self.enchant_dict = None
             elif IMPORT_HUNSPELL_SUCCESSFUL and self.dic_path:
                 aff_path = self.dic_path.replace('.dic', '.aff')
                 try:
                     self.pyhunspell_object = hunspell.HunSpell(
                         self.dic_path, aff_path)
-                except hunspell.HunSpellError:
+                except hunspell.HunSpellError as error:
                     LOGGER.debug(
-                        'Error initializing hunspell for %s', self.name)
+                        'Error initializing hunspell for %s: %s: %s',
+                        self.name, error.__class__.__name__, error)
                     self.pyhunspell_object = None
-                except Exception:
+                except Exception as error:
                     LOGGER.debug(
-                        'Unknown error initializing hunspell for %s',
-                        self.name)
+                        'Unexpected error initializing hunspell for '
+                        '%s: %s: %s',
+                        self.name, error.__class__.__name__, error)
                     self.pyhunspell_object = None
 
     def spellcheck_enchant(self, word: str) -> bool:
@@ -193,7 +202,7 @@ class Dictionary:
         if not self.voikko:
             return False
         # voikko works correctly if the input is a Unicode string in NFC.
-        return self.voikko.spell(unicodedata.normalize('NFC', word))
+        return bool(self.voikko.spell(unicodedata.normalize('NFC', word)))
 
     def spellcheck(self, word: str) -> bool:
         '''
@@ -221,7 +230,7 @@ class Dictionary:
         if self.pyhunspell_object:
             return self.spellcheck_pyhunspell(word)
         if self.voikko:
-            return self.voikko.spell(word)
+            return bool(self.voikko.spell(word))
         return False
 
     def has_spellchecking(self) -> bool:
@@ -341,7 +350,7 @@ class Hunspell:
     '''A class to suggest completions or corrections
     using a list of Hunspell dictionaries
     '''
-    def __init__(self, dictionary_names=()) -> None:
+    def __init__(self, dictionary_names: Iterable[str] = ()) -> None:
         global DEBUG_LEVEL
         try:
             DEBUG_LEVEL = int(
@@ -356,7 +365,7 @@ class Hunspell:
             else:
                 LOGGER.debug('Hunspell.__init__(dictionary_names=())\n')
         self._suggest_cache: Dict[str, List[Tuple[str, int]]] = {}
-        self._dictionary_names = dictionary_names
+        self._dictionary_names: List[str] = list(dictionary_names)
         self._dictionaries: List[Dictionary] = []
         self.init_dictionaries()
 
@@ -381,9 +390,9 @@ class Hunspell:
 
         It is important to return a copy, we do not want to change
         the private member variable directly.'''
-        return self._dictionary_names[:]
+        return list(self._dictionary_names[:])
 
-    def set_dictionary_names(self, dictionary_names: List[str]):
+    def set_dictionary_names(self, dictionary_names: List[str]) -> None:
         '''Sets the list of dictionary names.
 
         If the new list of dictionary names differs from the existing
@@ -466,6 +475,67 @@ class Hunspell:
             return False
         return True
 
+    def spellcheck_single_dictionary(self, words: Iterable[str] = ()) -> bool:
+        '''
+        Checks whether there is at least one dictionary where all words
+        in the input list spellcheck as True.
+
+        :param words: A list or tuple of words to spellcheck
+        :return: - False if at least one input word is empty
+                 - True if there is at least one dictionary where all
+                   words in the input list spellcheck as True, else False.
+
+        Examples:
+
+        >>> h = Hunspell(['en_US', 'fr_FR'])
+
+        All 3 words spellcheck True in the French dictionary:
+
+        >>> h.spellcheck_single_dictionary(('Je', 'suis', 'arrivé'))
+        True
+
+        Not a correct French sentence but all 3 words spellcheck True
+        in the French dictionary:
+
+        >>> h.spellcheck_single_dictionary(('Je', 'suis', 'arrive'))
+        True
+
+
+        Not all 3 words spellcheck True in the French dictionary and
+        not all 3 words spellcheck True in the English dictionary either:
+
+        >>> h.spellcheck_single_dictionary(('Je', 'suis', 'arrived'))
+        False
+
+        All 3 words spellcheck True in the English dictionary:
+
+        >>> h.spellcheck_single_dictionary(('I', 'have', 'arrived'))
+        True
+
+        Unfortunately these 3 words spellcheck True in the French dictionary
+        ('have' is a form of the verb 'haver', 'I' is the name of the letter):
+
+        >>> h.spellcheck_single_dictionary(('I', 'have', 'arrivé'))
+        True
+
+        If the input contains empty words the result is False:
+
+        >>> h.spellcheck_single_dictionary(('I', 'have', ''))
+        False
+        '''
+        if not words:
+            return False
+        for word in words:
+            if not word:
+                return False
+        for dictionary in self._dictionaries:
+            spellcheck_total = True
+            for word in words:
+                spellcheck_total &= dictionary.spellcheck(word)
+            if spellcheck_total:
+                return True
+        return False
+
     def suggest(self, input_phrase: str) -> List[Tuple[str, int]]:
         # pylint: disable=line-too-long
         '''Return completions or corrections for the input phrase
@@ -645,7 +715,7 @@ class Hunspell:
 
 BENCHMARK = True
 
-def main():
+def main() -> None:
     '''
     Used for testing and profiling.
 
