@@ -68,7 +68,7 @@ class TabSqliteDb:
         user_freq >= 1: The number of times the user has used this phrase
     '''
     # pylint: enable=line-too-long
-    def __init__(self, user_db_file='') -> None:
+    def __init__(self, user_db_file: str = '') -> None:
         global DEBUG_LEVEL
         try:
             DEBUG_LEVEL = int(str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
@@ -173,9 +173,10 @@ class TabSqliteDb:
                         LOGGER.info(
                             'Compatible database %(db)s found.',
                             {'db': self.user_db_file})
-                except Exception:
+                except Exception as error:
                     LOGGER.exception(
-                        'Unexpected error trying to find user database.')
+                        'Unexpected error searching user database: %s: %s',
+                        error.__class__.__name__, error)
 
         # open user phrase database
         try:
@@ -186,10 +187,10 @@ class TabSqliteDb:
             self.set_database_pragma_options()
             self.database.executescript(
                 'ATTACH DATABASE "%s" AS user_db;' % self.user_db_file)
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Could not open the database %(name)s.',
-                {'name': self.user_db_file})
+                'Could not open the database %s. %s: %s',
+                self.user_db_file, error.__class__.__name__, error)
             timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
             LOGGER.info(
                 'Renaming the incompatible database to "%(name)s".',
@@ -227,10 +228,11 @@ class TabSqliteDb:
             ;'''
             try:
                 self.database.executemany(sqlstr, sqlargs)
-            except Exception:
+            except Exception as error:
                 LOGGER.exception(
                     'Unexpected error inserting old phrases '
-                    'into the user database.')
+                    'into the user database: %s: %s',
+                    error.__class__.__name__, error)
             self.database.commit()
             self.database.execute('PRAGMA wal_checkpoint;')
 
@@ -275,8 +277,10 @@ class TabSqliteDb:
             self.database.execute(sqlstr, sqlargs)
             if commit:
                 self.database.commit()
-        except Exception:
-            LOGGER.exception('Unexpected error updating phrase in user_db.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error updating phrase in user_db: %s: %s',
+                error.__class__.__name__, error)
 
     def sync_usrdb(self) -> None:
         '''
@@ -355,8 +359,10 @@ class TabSqliteDb:
             self.database.execute(insert_sqlstr, insert_sqlargs)
             if commit:
                 self.database.commit()
-        except Exception:
-            LOGGER.exception('Unexpected error adding phrase to database.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error adding phrase to database: %s: %s',
+                error.__class__.__name__, error)
 
     def create_indexes(self, commit: bool = True) -> None:
         '''Create indexes for the database.'''
@@ -373,7 +379,7 @@ class TabSqliteDb:
     def best_candidates(
             self,
             phrase_frequencies: Dict[str, float],
-            title=False) -> List[Tuple[str, float]]:
+            title: bool = False) -> List[Tuple[str, float]]:
         '''Sorts the phrase_frequencies dictionary and returns the best
         candidates.
 
@@ -398,6 +404,44 @@ class TabSqliteDb:
             phrases_title.add(phrase_title)
         return candidates_title
 
+    def select_shortcuts(
+            self,
+            input_phrase: str) -> List[Tuple[str, float]]:
+        '''
+        Get shortcuts from database completing input_phrase.
+
+        Returns a list of matches where each match is a tuple in the
+        form of (phrase, user_freq), i.e. returns something like
+        [(phrase, user_freq), ...]
+        '''
+        input_phrase = unicodedata.normalize(
+            itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug('input_phrase=%s', input_phrase)
+        phrase_frequencies: Dict[str, float] = {}
+        sqlargs = {'input_phrase': input_phrase + '%',
+                   'user_freq': itb_util.SHORTCUT_USER_FREQ}
+        sqlstr = ('SELECT phrase, sum(user_freq) FROM user_db.phrases '
+                  'WHERE input_phrase LIKE :input_phrase '
+                  'AND user_freq >= :user_freq '
+                  'GROUP BY phrase;')
+        results_shortcuts: List[Tuple[str, int]] = []
+        try:
+            results_shortcuts = self.database.execute(
+                sqlstr, sqlargs).fetchall()
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error fetching '
+                'user shortcuts from database: %s: %s',
+                 error.__class__.__name__, error)
+        if results_shortcuts:
+            phrase_frequencies.update(results_shortcuts)
+        best_shortcut_candidates = self.best_candidates(phrase_frequencies)
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug(
+                'best_shortcut_candidates=%s', best_shortcut_candidates)
+        return best_shortcut_candidates
+
     def select_words(
             self,
             input_phrase: str,
@@ -466,8 +510,10 @@ class TabSqliteDb:
         ;''' % {'quoted_input_phrase': quoted_input_phrase}
         try:
             self.database.execute(sqlstr)
-        except Exception:
-            LOGGER.exception('Unexpected error in creating database view.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error in creating database view: %s: %s',
+                error.__class__.__name__, error)
         sqlargs = {'p_phrase': p_phrase, 'pp_phrase': pp_phrase}
         sqlstr = (
             'SELECT phrase, sum(user_freq) FROM like_input_phrase_view '
@@ -492,9 +538,10 @@ class TabSqliteDb:
             # (“c|conspiracy|1” is not selected because it doesn’t
             # match the user input “LIKE co%”! I.e. this is filtered
             # out by the VIEW created above already)
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting “unigram” data from user_db.')
+                'Unexpected error getting “unigram” data from user_db: %s: %s',
+                error.__class__.__name__, error)
         if not results_uni:
             # If no unigrams matched, bigrams and trigrams cannot
             # match either. We can stop here and return what we got
@@ -507,9 +554,11 @@ class TabSqliteDb:
         sqlstr = 'SELECT sum(user_freq) FROM like_input_phrase_view;'
         try:
             count = self.database.execute(sqlstr, sqlargs).fetchall()[0][0]
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting total unigram count from user_db')
+                'Unexpected error getting total unigram count '
+                'from user_db: %s: %s',
+                 error.__class__.__name__, error)
         # Updating the phrase_frequency dictionary with the normalized
         # results gives: {'conspiracy': 6/11, 'code': 0,
         # 'communicability': 0, 'cold': 1/11, 'colour': 4/11}
@@ -529,9 +578,11 @@ class TabSqliteDb:
             'WHERE p_phrase = :p_phrase GROUP BY phrase;')
         try:
             results_bi = self.database.execute(sqlstr, sqlargs).fetchall()
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting “bigram” data from user_db')
+                'Unexpected error getting “bigram” data '
+                'from user_db: %s: %s',
+                error.__class__.__name__, error)
         if not results_bi:
             # If no bigram could be matched, return what we have so far:
             return self.best_candidates(phrase_frequencies, title=title_case)
@@ -542,9 +593,11 @@ class TabSqliteDb:
         try:
             count_p_phrase = self.database.execute(
                 sqlstr, sqlargs).fetchall()[0][0]
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting total bigram count from user_db')
+                'Unexpected error getting total bigram count '
+                'from user_db: %s: %s',
+                 error.__class__.__name__, error)
         # Update the phrase frequency dictionary by using a linear
         # combination of the unigram and the bigram results, giving
         # both the weight of 0.5:
@@ -566,9 +619,11 @@ class TabSqliteDb:
                   'AND pp_phrase = :pp_phrase GROUP BY phrase;')
         try:
             results_tri = self.database.execute(sqlstr, sqlargs).fetchall()
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting “trigram” data from user_db')
+                'Unexpected error getting “trigram” data '
+                'from user_db: %s: %s',
+                 error.__class__.__name__, error)
         if not results_tri:
             # if no trigram could be matched, return what we have so far:
             return self.best_candidates(phrase_frequencies, title=title_case)
@@ -580,9 +635,11 @@ class TabSqliteDb:
         try:
             count_pp_phrase_p_phrase = self.database.execute(
                 sqlstr, sqlargs).fetchall()[0][0]
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting total trigram count from user_db')
+                'Unexpected error getting total trigram count '
+                'from user_db: %s: %s',
+                 error.__class__.__name__, error)
         # Update the phrase frequency dictionary by using a linear
         # combination of the bigram and the trigram results, giving
         # both the weight of 0.5 (that makes the total weights: 0.25 *
@@ -617,8 +674,10 @@ class TabSqliteDb:
                 + 'VALUES (?, DATETIME("now", "localtime"));')
             self.database.execute(sqlstring, ("create-time", ))
             self.database.commit()
-        except Exception:
-            LOGGER.exception('Unexpected error adding description to user_db.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error adding description to user_db: %s: %s',
+                 error.__class__.__name__, error)
 
     def init_user_db(self) -> None:
         '''
@@ -660,8 +719,10 @@ class TabSqliteDb:
                 desc[row[0]] = row[1]
             database.close()
             return desc
-        except Exception:
-            LOGGER.exception('Unexpected error getting database description.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error getting database description: %s: %s',
+                 error.__class__.__name__, error)
             return None
 
     def get_number_of_columns_of_phrase_table(
@@ -696,9 +757,11 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                 table_phrases_columns = match.group(1).split(',')
                 return len(table_phrases_columns)
             return 0
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error getting number of columns of database.')
+                'Unexpected error getting number of columns '
+                'of database: %s: %s',
+                 error.__class__.__name__, error)
             return 0
 
     def list_user_shortcuts(self) -> List[Tuple[str, str]]:
@@ -716,6 +779,81 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
             LOGGER.debug('result=%s', result)
         return result
 
+    def check_shortcut_and_update_frequency(
+            self,
+            input_phrase: str = '',
+            phrase: str = '',
+            user_freq_increment: int = 1,
+            commit: bool = True) -> bool:
+        '''
+        Check whether input_phrase and phrase are a user defined shortcut.
+
+        Return True if this was a shortcut, else return False.
+
+        If this is already saved in the database as a shortcut,
+        increase the frequency by 1.
+
+        If no such shortcut is in the database but user_freq_increment
+        is >= itb_util.SHORTCUT_USER_FREQ, add such a shortcut.
+        '''
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug(
+                'input_phrase=%s phrase=%s user_freq_increment=%s commit=%s',
+                input_phrase, phrase, user_freq_increment, commit)
+        if not input_phrase or not phrase:
+            return False
+        phrase = unicodedata.normalize(
+            itb_util.NORMALIZATION_FORM_INTERNAL, phrase)
+        input_phrase = unicodedata.normalize(
+            itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
+        sqlargs = {'input_phrase': input_phrase,
+                   'phrase': phrase,
+                   'user_freq': itb_util.SHORTCUT_USER_FREQ,
+                   'timestamp': time.time()}
+        sqlstr = ('SELECT max(user_freq) FROM user_db.phrases '
+                  'WHERE input_phrase = :input_phrase '
+                  'AND phrase = :phrase '
+                  'AND user_freq >= :user_freq '
+                  'GROUP BY phrase;')
+        result =  self.database.execute(sqlstr, sqlargs).fetchall()
+        if result:
+            if DEBUG_LEVEL > 1:
+                LOGGER.debug('Existing shortcut, increase frequency by 1.')
+            sqlargs['user_freq'] = result[0][0] + 1
+            sqlstr = (
+                'UPDATE user_db.phrases '
+                'SET user_freq = :user_freq, timestamp = :timestamp '
+                'WHERE input_phrase = :input_phrase AND phrase = :phrase ;')
+            try:
+                self.database.execute(sqlstr, sqlargs)
+                if commit:
+                    self.database.commit()
+            except Exception as error:
+                LOGGER.exception(
+                    'Unexpected error updating shortcut in user_db: %s: %s',
+                     error.__class__.__name__, error)
+            return True
+        if user_freq_increment < itb_util.SHORTCUT_USER_FREQ:
+            if DEBUG_LEVEL > 1:
+                LOGGER.debug('It was not a shortcut, return False.')
+            return False
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug('New shortcut, add it to the database.')
+        sqlargs['user_freq'] = user_freq_increment
+        sqlstr = (
+            'INSERT INTO user_db.phrases '
+            '(input_phrase, phrase, p_phrase, pp_phrase, user_freq, timestamp)'
+            'VALUES (:input_phrase, :phrase, "", "", :user_freq, :timestamp);')
+        try:
+            self.database.execute(sqlstr, sqlargs)
+            if commit:
+                self.database.commit()
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error adding shortcut to database: %s: %s',
+                error.__class__.__name__, error)
+        return True
+
     def check_phrase_and_update_frequency(
             self,
             input_phrase: str = '',
@@ -729,21 +867,32 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         they are in the database, increase the frequency by 1, if not
         add them.
         '''
+        if DEBUG_LEVEL > 1:
+            LOGGER.debug(
+                'input_phrase=%s phrase=%s p_phrase=%s pp_phrase=%s '
+                'user_freq_increment=%s commit=%s',
+                input_phrase, phrase, p_phrase, pp_phrase,
+                user_freq_increment, commit)
+        # Handle shortcuts before using strip_token to allow
+        # punctuation in shortcuts and shortcut expansions:
+        if self.check_shortcut_and_update_frequency(
+                input_phrase=input_phrase,
+                phrase=phrase,
+                user_freq_increment=user_freq_increment,
+                commit=commit):
+            return
         if not input_phrase:
             input_phrase = phrase
         if not phrase:
             return
+        input_phrase = itb_util.strip_token(input_phrase)
+        phrase = itb_util.strip_token(phrase)
         phrase = unicodedata.normalize(
             itb_util.NORMALIZATION_FORM_INTERNAL, phrase)
         p_phrase = itb_util.remove_accents(p_phrase.lower())
         pp_phrase = itb_util.remove_accents(pp_phrase.lower())
         input_phrase = itb_util.remove_accents(input_phrase.lower())
 
-        if DEBUG_LEVEL > 1:
-            LOGGER.debug(
-                'phrase=%(p)s, input_phrase=%(t)s',
-                {'p': phrase, 't': input_phrase})
-
         # There should never be more than 1 database row for the same
         # input_phrase *and* phrase. So the following query on
         # the database should match at most one database
@@ -846,8 +995,10 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                 phrases
             ]
             return phrases[:]
-        except Exception:
-            LOGGER.exception('Unexpected error extracting user phrases.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error extracting user phrases: %s: %s',
+                 error.__class__.__name__, error)
             return []
 
     def read_training_data_from_file(self, filename: str) -> bool:
@@ -860,13 +1011,13 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
             filename += '.gz'
             if not os.path.isfile(filename):
                 return False
-        open_function: Callable = open
+        open_function: Callable[[Any], Any] = open
         if filename.endswith('.gz'):
             open_function = gzip.open
         rows = self.database.execute(
             'SELECT input_phrase, phrase, p_phrase, pp_phrase, '
             + 'user_freq, timestamp FROM phrases;').fetchall()
-        rows = sorted(rows, key = lambda x: (x[5])) # sort by timestamp
+        rows = sorted(rows, key = lambda x: (float(x[5]))) # sort by timestamp
         time_min = time.time()
         time_max = time.time()
         if rows:
@@ -902,15 +1053,16 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                                   )])
         lines = []
         try:
-            with open_function(
+            with open_function( # type: ignore
                     filename, mode='rt', encoding='UTF-8') as file_handle:
                 lines = [
                     unicodedata.normalize(
                         itb_util.NORMALIZATION_FORM_INTERNAL, line)
                     for line in file_handle.readlines()]
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error reading training data from file.')
+                'Unexpected error reading training data from file: %s: %s',
+                 error.__class__.__name__, error)
             return False
         for line in lines:
             for token in itb_util.tokenize(line):
@@ -946,9 +1098,10 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
             self.database.executemany(sqlstr, sqlargs)
             self.database.commit()
             self.database.execute('PRAGMA wal_checkpoint;')
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error writing training data to database.')
+                'Unexpected error writing training data to database: %s: %s',
+                 error.__class__.__name__, error)
             return False
         return True
 
@@ -961,9 +1114,10 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
             self.database.execute('DELETE FROM phrases;')
             self.database.commit()
             self.database.execute('PRAGMA wal_checkpoint;')
-        except Exception:
+        except Exception as error:
             LOGGER.exception(
-                'Unexpected error removing all phrases from database.')
+                'Unexpected error removing all phrases from database: %s: %s',
+                 error.__class__.__name__, error)
 
     def dump_database(self) -> None:
         '''
@@ -979,8 +1133,9 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
             for row in self.database.execute(
                     "SELECT * FROM phrases;").fetchall():
                 LOGGER.debug('%s', repr(row))
-        except Exception:
-            LOGGER.exception('Unexpected error dumping database.')
+        except Exception as error:
+            LOGGER.exception('Unexpected error dumping database: %s: %s',
+                              error.__class__.__name__, error)
             return
 
     def number_of_rows_in_database(self) -> int:
@@ -992,8 +1147,10 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         try:
             return len(self.database.execute(
                 "SELECT * FROM phrases;").fetchall())
-        except Exception:
-            LOGGER.exception('Unexpected error getting number of database rows.')
+        except Exception as error:
+            LOGGER.exception(
+                'Unexpected error getting number of database rows: %s: %s',
+                error.__class__.__name__, error)
             return -1
 
     def cleanup_database(self, thread: bool = True) -> None:
@@ -1026,8 +1183,9 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
             else:
                 database = self.database
             rows = database.execute("SELECT * FROM phrases;").fetchall()
-        except Exception:
-            LOGGER.exception('Exception when accessing database')
+        except Exception as error:
+            LOGGER.exception('Exception when accessing database: %s: %s',
+                              error.__class__.__name__, error)
             import traceback
             traceback.print_exc()
         if not rows:
@@ -1056,9 +1214,11 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                 sqlargs_delete = {'id': row[0]}
                 try:
                     database.execute(sqlstr_delete, sqlargs_delete)
-                except Exception:
+                except Exception as error:
                     LOGGER.exception(
-                        '1st pass: exception deleting row from database.')
+                        '1st pass: exception deleting row '
+                        'from database: %s: %s',
+                         error.__class__.__name__, error)
                     import traceback
                     traceback.print_exc()
             else:
@@ -1107,9 +1267,11 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
                     sqlargs_delete = {'id': row[0]}
                     try:
                         database.execute(sqlstr_delete, sqlargs_delete)
-                    except Exception:
+                    except Exception as error:
                         LOGGER.exception(
-                            '2nd pass: exception deleting row from database.')
+                            '2nd pass: exception deleting row '
+                            'from database: %s: %s',
+                             error.__class__.__name__, error)
                         import traceback
                         traceback.print_exc()
                 else:
@@ -1129,9 +1291,11 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
 
                     try:
                         database.execute(sqlstr_update, sqlargs_update)
-                    except Exception:
+                    except Exception as error:
                         LOGGER.exception(
-                            '2nd pass: exception decaying row from database.')
+                            '2nd pass: exception decaying row '
+                            'from database: %s: %s',
+                             error.__class__.__name__, error)
                         import traceback
                         traceback.print_exc()
             index -= 1
